import time
import board
import neopixel_spi as neopixel
from gpiozero import LED

LED_R = LED(23)
LED_L = LED(24)
BZ = LED(18)

LED_R.off()
LED_L.off()
BZ.off()

NUM_PIXELS = 5
PIXEL_ORDER = neopixel.GRB
COLORS = (0xFF0000, 0x00FF00, 0x0000FF)
DELAY = 0.02

spi = board.SPI()

pixels = neopixel.NeoPixel_SPI(
    spi, NUM_PIXELS, pixel_order=PIXEL_ORDER, auto_write=False
)

class RGBW(int):
    def __new__(self, r, g=None, b=None, w=None):
        if (g,b,w) == (None, None, None):
            return int.__new__(self, r)
        else:
            if w is None:
                w = 0
            return int.__new__(self, (w << 24) | (r << 16) | (g << 8) | b)
    @property
    def r(self):
        return (self >> 16) & 0xFF
    @property
    def g(self):
        return (self >> 8) & 0xFF
    @property
    def b(self):
        return (self) & 0xFF
    @property
    def w(self):
        return (self >> 24) & 0xFF
        

def Color(red, green, blue, white=0):
    return RGBW(red, green, blue, white)

def colorWipe(strip, color, wait_ms=5):
    """Wipe color across display a pixel at a time."""
    for i in range(strip.n):
        strip[i] = color
        strip.show()
        time.sleep(wait_ms/ 1000.0)

try:
    while True:
        LED_R.on()
        LED_L.on()
        BZ.on()
        for i in range(5):
            colorWipe(pixels, Color(0, 0, 0))
            pixels[i] = Color(128, 0, 0)
            pixels.show()
            time.sleep(0.1)
        LED_R.off()
        LED_L.off()
        BZ.off()
        for i in range(3, 0, -1):
            colorWipe(pixels, Color(0, 0, 0))
            pixels[i] = Color(128, 0, 0)
            pixels.show()
            time.sleep(0.1)
        time.sleep(0.05)

except KeyboardInterrupt:
    colorWipe(pixels, (0,0,0), 10)
        
        
