import cv2
import ollama
import json
import re
import threading
import subprocess

import time
import board
import neopixel_spi as neopixel
from gpiozero import LED
import adafruit_bme680

i2c = board.I2C()
bme680 = adafruit_bme680.Adafruit_BME680_I2C(i2c, debug=False)
bme680.sea_level_pressure = 1013.25
temperature_offset = 0


LED_R = LED(23)
LED_L = LED(24)
BZ = LED(18)
LED_R.off()
LED_L.off()
BZ.off()

NUM_PIXELS = 5
PIXEL_ORDER = neopixel.GRB
COLORS = (0xFF0000, 0x00FF00, 0x0000FF)
DELAY = 0.02
spi = board.SPI()

pixels = neopixel.NeoPixel_SPI(spi, NUM_PIXELS, pixel_order=PIXEL_ORDER, auto_write=False)

class RGBW(int):
    def __new__(self, r, g=None, b=None, w=None):
        if (g,b,w) == (None, None, None):
            return int.__new__(self, r)
        else:
            if w is None:
                w = 0
            return int.__new__(self, (w << 24) | (r << 16) | (g << 8) | b)
    @property
    def r(self):
        return (self >> 16) & 0xFF
    @property
    def g(self):
        return (self >> 8) & 0xFF
    @property
    def b(self):
        return (self) & 0xFF
    @property
    def w(self):
        return (self >> 24) & 0xFF


def Color(red, green, blue, white=0):
    return RGBW(red, green, blue, white)

def colorWipe(strip, color, wait_ms=5):
    for i in range(strip.n):
        strip[i] = color
        strip.show()
        time.sleep(wait_ms/ 1000.0)

def knightrider():
    while True:
        for i in range(5):
            colorWipe(pixels, Color(0, 0, 0))
            pixels[i] = Color(128, 0, 0)
            pixels.show()
            time.sleep(0.1)
        for i in range(3, 0, -1):
            colorWipe(pixels, Color(0, 0, 0))
            pixels[i] = Color(128, 0, 0)
            pixels.show()
            time.sleep(0.1)
        time.sleep(0.05)


def speak_ojtalk(text):
    open_jtalk = ['open_jtalk']
    mecab_dict = ['-x','/var/lib/mecab/dic/open-jtalk/naist-jdic']
    htsvoice = ['-m','/usr/share/hts-voice/nitech-jp-atr503-m001/nitech_jp_atr503_m001.htsvoice']

    speed = ['-r','1.2']
    outwav = ['-ow','test.wav']
    cmd = open_jtalk+mecab_dict+htsvoice+speed+outwav
    c = subprocess.Popen(cmd,stdin=subprocess.PIPE)
    c.stdin.write(text.encode())
    c.stdin.close()
    c.wait()
    aplay = ['aplay','-q','test.wav']
    wr = subprocess.Popen(aplay)


def extract_json_from_string(input_string):
    json_pattern = re.compile(r'\{.*?\}')
    input_string = input_string.replace("\n", "")
    json_matches = re.findall(json_pattern, input_string)
    json_data_string = ''.join(json_matches)
    return json_data_string

knight_thread = threading.Thread(target=knightrider , daemon=True)
knight_thread.start()

def get_environment_data():
    temp = 0
    gas  = 0
    hum  = 0
    pres = 0
    alt  = 0
    for i in range(10):
        temp = temp + bme680.temperature + temperature_offset
        gas  = gas  + bme680.gas
        hum  = hum  + bme680.relative_humidity
        pres = pres + bme680.pressure
        alt  = alt  + bme680.altitude
    temp = round(temp/10,1)
    gas  = round(gas /10,1)
    hum  = round(hum /10,1)
    pres = round(pres/10,3)
    alt  = round(alt /10,2)
    print("温度: %0.1f C" % temp)
    print("ガス濃度: %d ohm" % gas)
    print("湿度: %0.1f %%" % hum)
    print("気圧: %0.3f hPa" % pres)
    print("高度: %0.2f meters" % alt)
    return "この画像の部屋の温度は" + str(temp) + "℃です。湿度は" + str(hum)+ "%です。この部屋は危険ですか安全ですか？" 


content_text1 = """
あなたはパワフルなスポーツカーに搭載されたAIです。
あなたとパートナーのマイケルは、さまざまな事件を解決し、人々を助けます。
"""
content_text3 = """
煙や火があったり、棒、ハンマー、銃、ナイフなどの武器を持っている人がいたら、危険であると判断して警告を出力してください。
出力はJSON形式です。
「people」キーは画像に見える人間の人数を示します。
「warn」キーは、最小0から最大5までの危険度を示します。最も安全な場合は0に設定します。
何か注意をした方が良い場合は危険度に応じて1から3を設定します。警告が必要な場合は4を設定します。
火事が起きている時、ナイフ、ハンマー、武器を持った危険人物など、危害を加えられる恐れがある場合は5に設定します。
「comment」キーは、画像から判断した警告内容を日本語でマイケルに出力します。

必ずJSON形式で出力してください　以下に出力形式を示す
{"people":1,"warn":5,"comment": "message"}
"""


capture = cv2.VideoCapture(0)
capture.set(cv2.CAP_PROP_BUFFERSIZE, 1)

while True:
    LED_R.on()
    LED_L.on()
    content_text2 = get_environment_data()    
    content_text = content_text1 + content_text2 + content_text3
    print(content_text)
    dmy = frame = capture.read()
    ret, frame = capture.read()
    cv2.imwrite("capture_image.jpg", frame )
    LED_R.off()
    LED_L.off()

    res = ollama.chat(
        model="llava",
        messages=[{
            'role': 'user',
            'content':content_text,
            'images': ['./capture_image.jpg']
        }]
    )

    out_text = res['message']['content']
    print(out_text)
    result = extract_json_from_string(out_text)

    try:
        output = json.loads(result)
        print(output["people"])
        print(output["warn"])
        print(output["comment"])
        speak_ojtalk(output["comment"])
        if output["warn"] == 5:
            LED_R.on()
            LED_L.on()
            BZ.on()
            time.sleep(10)
            LED_R.off()
            LED_L.off()            
            BZ.off()
        else:
            for i in range (output["warn"]):
                LED_R.on()
                LED_L.on()
                time.sleep(0.5)
                LED_R.off()
                LED_L.off()               

    except Exception as e:
        print(e)
        print("JSON解析エラー")
